unit synthmain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, AudioIO, ExtCtrls, ComCtrls, FBVUMeter, DrawBox;

type

  psample = ^smallint;

  TMainForm = class(TForm)
    AudioIn1: TAudioIn;
    Button1: TButton;
    AudioOut1: TAudioOut;
    Label1: TLabel;
    Label2: TLabel;
    Button2: TButton;
    Button3: TButton;
    FBVUMeter1: TFBVUMeter;


    function AudioIn1BufferFilled(Buffer: PChar;
      var Size: Integer): Boolean;
    procedure Button2Click(Sender: TObject);
    function AudioOut1FillBuffer(Buffer: PChar;
      var Size: Integer): Boolean;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button1MouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Button1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure Button3Click(Sender: TObject);
  private
    { Private declarations }
    buf_count: integer;
    sample_buffer: psample;
    sample_count: integer;
    out_count: integer;

    recp: psample;
    playp: psample;
    rec: boolean;
    play: boolean;
    reverse: boolean;

    procedure OpenAudio;
    procedure CloseAudio;
  public
    { Public declarations }
  end;

const
  buf_size = 44100 * 100;

var
  MainForm: TMainForm;

implementation

{$R *.DFM}

function TMainForm.AudioIn1BufferFilled(Buffer: PChar;
  var Size: Integer): Boolean;
var
  i: integer;
  max, min: smallint;
  p: psample;
begin
  if rec then
  begin
    p := psample(buffer);
    for i := 0 to (size div 2) - 1 do
    begin
      if sample_count < buf_size then
      begin
        recp^ := p^;
        inc(sample_count);
        inc(p);
        inc(recp);
      end
      else
        rec := false;
    end;
  end;
  p := psample(buffer);

  // determine max for VU meter
  max := 0;
  for i := 0 to (size div 2) - 1 do
  begin
    if p^ > max then max := p^;
    inc(p);
  end;
  FBVUMeter1.SetValue(max / 32000, max / 32000);
end;

procedure TMainForm.OpenAudio;
begin
  AudioOut1.Paused := True;
  if not AudioOut1.Start then
  begin
    application.messageBox('No audio device opened', 'open audio device', mb_OK);
  end
  else
    AudioOut1.Paused := False;
end;

procedure TMainForm.CloseAudio;
begin
end;

procedure TMainForm.Button2Click(Sender: TObject);
begin
  playp := sample_buffer;
  out_count := 0;
  reverse := false;
  play := true;
end;

function TMainForm.AudioOut1FillBuffer(Buffer: PChar;
  var Size: Integer): Boolean;
var
  p: psample;
  i: integer;
begin
  if play then
  begin
    p := psample(buffer);
    for i := 0 to (size div 4) - 1 do
    begin
      if out_count < sample_count then
      begin
        p^ := playp^;
        inc(p);
        p^ := playp^;
        inc(p);
        inc(out_count);
        if reverse then dec(playp)else inc(playp);
      end
      else
      begin
        p^ := 0;
        play := false;
      end;
    end;
    label1.caption := IntToStr(out_count);
    label1.update;
  end
  else
  begin
    p := psample(buffer);
    for i := 0 to (size div 2) - 1 do
    begin
        p^ := 0;
        inc(p);
    end;
  end;
  result := true;
end;

procedure TMainForm.FormCreate(Sender: TObject);
begin
  sample_buffer := nil;
  rec := false;
  Getmem(sample_buffer, buf_size * 2);
  Audioin1.start;
  openaudio;
end;

procedure TMainForm.FormDestroy(Sender: TObject);
begin
  if not assigned(sample_buffer) then FreeMem(sample_buffer);
end;

procedure TMainForm.Button1MouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  recp := sample_buffer;
  sample_count := 0;
  rec := true;
end;

procedure TMainForm.Button1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  rec := false;
  label2.caption := inttostr(sample_Count);
end;

procedure TMainForm.Button3Click(Sender: TObject);
begin
  playp := recp;
  out_count := 0;
  reverse := true;
  play := true;
end;

end.

